package xyz.wbsite.endpoint;

import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Controller;

import javax.websocket.CloseReason;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;

@ServerEndpoint("/websocket/{userId}")
@Controller
public class WebSocketManager {

    static Log log = LogFactory.getLog(WebSocketManager.class);

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    private static ConcurrentHashMap<String, Session> keySessionMap = new ConcurrentHashMap<>();
    private static ConcurrentHashMap<Integer, String> hashKeyMap = new ConcurrentHashMap<>();

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("userId") String userId) {
        // 移除并关闭旧会话
        if (keySessionMap.containsKey(userId)) {
            Session remove = keySessionMap.remove(userId);
            hashKeyMap.remove(remove.hashCode());
            try {
                remove.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        keySessionMap.put(userId, session);
        hashKeyMap.put(session.hashCode(), userId);

        log.info("用户连接:" + userId + ",当前在线人数为:" + keySessionMap.size());

        sendMessage(userId, "连接成功");
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session) {
        if (hashKeyMap.containsKey(session.hashCode())) {
            String userId = hashKeyMap.remove(session.hashCode());
            keySessionMap.remove(userId);
            log.info("用户退出:" + userId);
        }
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        log.info("收到客户端消息:" + message);
    }

    /**
     * 发生错误移除会话
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        error.printStackTrace();
        if (hashKeyMap.containsKey(session.hashCode())) {
            String userId = hashKeyMap.remove(session.hashCode());
            keySessionMap.remove(userId);
            log.error("用户错误:" + userId + ",原因:" + error.getMessage());
        }
    }

    /**
     * 实现服务器主动推送
     */
    public void sendMessage(String userId, String message) {
        Session session = keySessionMap.get(userId);
        if (session == null) {
            return;
        }
        try {
            session.getBasicRemote().sendText(message);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Scheduled(fixedDelay = 3 * 1000)
    public void sendTask() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        sendMessage("1", simpleDateFormat.format(new Date()));
    }
}
